@@ -27,7 +27,7 @@
 ****************************************************************************/
 
 
-#include <QtTest/QtTest>
+#include <QTest>
 #include <qlayout.h>
 #include <qapplication.h>
 #include <qwidget.h>
@@ -235,6 +235,9 @@ void tst_QGridLayout::badDistributionBug()
 
 void tst_QGridLayout::setMinAndMaxSize()
 {
+    if (QGuiApplication::platformName().startsWith(QLatin1String("wayland"), Qt::CaseInsensitive))
+        QSKIP("This test crashes on Wayland, see also QTBUG-107184");
+
     QWidget widget;
     setFrameless(&widget);
     QGridLayout layout(&widget);
@@ -382,12 +385,12 @@ void tst_QGridLayout::setMinAndMaxSize()
 class SizeHinter : public QWidget
 {
 public:
-    SizeHinter(const QSize &s, QWidget *parent = 0)
+    SizeHinter(const QSize &s, QWidget *parent = nullptr)
         : QWidget(parent), sh(s) { }
-    SizeHinter(int w, int h, QWidget *parent = 0)
+    SizeHinter(int w, int h, QWidget *parent = nullptr)
         : QWidget(parent), sh(QSize(w,h)) {}
     void setSizeHint(QSize s) { sh = s; }
-    QSize sizeHint() const { return sh; }
+    QSize sizeHint() const override { return sh; }
 private:
     QSize sh;
 };
@@ -450,33 +453,33 @@ class Qt42Style : public QProxyStyle
 public:
     Qt42Style() : QProxyStyle(QStyleFactory::create("windows"))
     {
-        spacing = 6;
-        margin = 9;
-        margin_toplevel = 11;
     }
 
-    virtual int pixelMetric(PixelMetric metric, const QStyleOption * option = 0,
-                            const QWidget * widget = 0 ) const;
+    virtual int pixelMetric(PixelMetric metric, const QStyleOption * option = nullptr,
+                            const QWidget * widget = nullptr ) const override;
 
-    int spacing;
-    int margin;
-    int margin_toplevel;
+    int spacing = 6;
+    int margin = 9;
+    int margin_toplevel = 11;
 
 };
 
-int Qt42Style::pixelMetric(PixelMetric metric, const QStyleOption * option /*= 0*/,
-                                   const QWidget * widget /*= 0*/ ) const
+int Qt42Style::pixelMetric(PixelMetric metric, const QStyleOption * option,
+                           const QWidget * widget) const
 {
     switch (metric) {
-        case PM_DefaultLayoutSpacing:
-            return spacing;
-        break;
-        case PM_DefaultTopLevelMargin:
-            return margin_toplevel;
-        break;
-        case PM_DefaultChildMargin:
+        case PM_LayoutLeftMargin:
+        case PM_LayoutRightMargin:
+        case PM_LayoutTopMargin:
+        case PM_LayoutBottomMargin:
+            if (option && option->state & State_Window)
+                return margin_toplevel;
+            if (widget && widget->isWindow())
+                return margin_toplevel;
             return margin;
-        break;
+        case PM_LayoutHorizontalSpacing:
+        case PM_LayoutVerticalSpacing:
+            return spacing;
         default:
             break;
     }
@@ -490,7 +493,7 @@ typedef QList<QPoint> PointList;
 class SizeHinterFrame : public QLabel
 {
 public:
-    SizeHinterFrame(QWidget *parent = 0)
+    SizeHinterFrame(QWidget *parent = nullptr)
     : QLabel(parent)
     {
         init(-1);
@@ -509,11 +512,11 @@ public:
     }
 
     void setSizeHint(const QSize &s) { sh = s; }
-    QSize sizeHint() const { return sh; }
+    QSize sizeHint() const override { return sh; }
     void setMinimumSizeHint(const QSize &s) { msh = s; }
-    QSize minimumSizeHint() const { return msh; }
+    QSize minimumSizeHint() const override { return msh; }
 
-    virtual int heightForWidth(int width) const;
+    virtual int heightForWidth(int width) const override;
 
     void setNumberOfPixels(int numPixels) {
         m_numPixels = numPixels;
@@ -901,10 +904,10 @@ public:
         reimplementSubelementRect = false;
     }
 
-    virtual int pixelMetric(PixelMetric metric, const QStyleOption * option = 0,
-                            const QWidget * widget = 0 ) const;
+    virtual int pixelMetric(PixelMetric metric, const QStyleOption * option = nullptr,
+                            const QWidget * widget = nullptr ) const override;
     virtual QRect subElementRect(SubElement sr, const QStyleOption *opt,
-                                const QWidget *widget) const;
+                                const QWidget *widget) const override;
 
     int hspacing;
     int vspacing;
@@ -913,8 +916,8 @@ public:
     int layoutSpacing(QSizePolicy::ControlType control1,
                       QSizePolicy::ControlType control2,
                       Qt::Orientation orientation,
-                      const QStyleOption *option = 0,
-                      const QWidget *widget = 0) const;
+                      const QStyleOption *option = nullptr,
+                      const QWidget *widget = nullptr) const override;
 
 };
 
@@ -946,8 +949,8 @@ QRect CustomLayoutStyle::subElementRect(SubElement sr, const QStyleOption *opt,
 int CustomLayoutStyle::layoutSpacing(QSizePolicy::ControlType control1,
                                 QSizePolicy::ControlType control2,
                                 Qt::Orientation orientation,
-                                const QStyleOption * /*option = 0*/,
-                                const QWidget * /*widget = 0*/) const
+                                const QStyleOption * /*option = nullptr*/,
+                                const QWidget * /*widget = nullptr*/) const
 {
     if (orientation == Qt::Horizontal) {
         switch (CT2(control1, control2)) {
@@ -967,8 +970,8 @@ int CustomLayoutStyle::layoutSpacing(QSizePolicy::ControlType control1,
     }
 }
 
-int CustomLayoutStyle::pixelMetric(PixelMetric metric, const QStyleOption * option /*= 0*/,
-                                   const QWidget * widget /*= 0*/ ) const
+int CustomLayoutStyle::pixelMetric(PixelMetric metric, const QStyleOption * option /*= nullptr*/,
+                                   const QWidget * widget /*= nullptr*/ ) const
 {
     switch (metric) {
         case PM_LayoutLeftMargin:
